# *-* coding=utf8 *-*
# ******************************************************************************
# helper.py
# ------------------------------------------------------------------------------
#
# Copyright (c) 2018 JSH <downloader@gmx.de>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# ******************************************************************************
import sqlite3
import pandas as pd
import matplotlib.pyplot as plt

    
def scatterdiagramm(x, y, xlabel, ylabel):
    """Erzeugt ein Scatterdiagramm mit den übergebenen Werten

    :param x: pd.Series
    :param y: pd.Series
    :param xlabel: Beschreibung der x-Achse
    :param ylabel: Beschreibung der y-Achse
    """
    plt.scatter(x=x, y=y)
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.show()    


def kreisdiagramm_positives_drehmoment(df, filter):
    """Erzeugt ein Kreisdiagramm der positiven Drehmomente

    :param df: pd.DataFrame
    :param filter: Einschränkung im DataFrame
    """
    _kreisdiagramm([
        len((df[(df.MG2_TORQUE > 0) & filter])),
        len((df[(df.MGR_TORQUE > 0) & filter]))
    ],
        ['MG2', 'MGR']
    )


def kreisdiagramm_negatives_drehmoment(df, filter):
    """Erzeugt ein Kreisdiagramm der negativen Drehmomente

    :param df: pd.DataFrame
    :param filter: Einschränkung im DataFrame
    """
    _kreisdiagramm([
        len((df[(df.MG2_TORQUE < 0) & filter])),
        len((df[(df.MGR_TORQUE < 0) & filter]))
    ],
        ['MG2', 'MGR']
    )


def add_geschwindigkeitsaenderung(df):
    """Erweitert das DataFrame um die Spalte GESCHW

    :param df: DataFrame
    """
    result = []
    vorgaenger = 0.0
    for i in df.SPEED_OBD:
        if i > vorgaenger:
            result.append("steigt")
        elif i == vorgaenger:
            result.append("gleich")
        else:
            result.append("fällt")
        vorgaenger = i
    df['GESCHW'] = pd.Series(result, index=df.index)    


def ermittel_werte_bei_positiven_drehmoment(df, filter):
    """Gibt ein paar Zusatzinformationen bei positiven Drehmoment aus

    :param df: DataFrame
    :param filter: Einschränkung im DataFrame
    """
    msg = "\n".join([
        "Folgende Werte wurden bei positiven Drehmoment am MGR ermittelt:",
        "   - maximale Geschwindigkeit: {} km/h".format(max(df[filter].SPEED_OBD)),
        "   - maximales Drehmoment MGR: {} Nm".format(max(df[filter].MGR_TORQUE)),
        "   - maximales Drehmoment MG2: {} Nm".format(max(df[filter].MG2_TORQUE))
    ])
    print(msg)    
    
    
def ermittel_werte_bei_negativen_drehmoment(df, filter):
    """Gibt ein paar Zusatzinformationen bei negativen Drehmoment aus

    :param df: DataFrame
    :param filter: Einschränkung im DataFrame
    """
    msg = "\n".join([
        "Folgende Werte wurden bei negativen Drehmoment am MGR ermittelt:",
        "   - maximale Geschwindigkeit: {} km/h".format(max(df[filter].SPEED_OBD)),
        "   - minimales Drehmoment MGR: {} Nm".format(min(df[filter].MGR_TORQUE)),
        "   - minimales Drehmoment MG2: {} Nm".format(min(df[filter].MG2_TORQUE))
    ])
    print(msg)    
 

def _kreisdiagramm(values, labels):
    """Erzeugt ein Kreisdiagramm

    :param values: Liste mit Werten
    :param labels: Liste mit Labels
    """
    plt.pie(values, labels=labels, autopct='%1.1f%%')
    plt.axis("equal")
    plt.show()        